/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Aufgaben mit mehreren Teilaufgaben, bei der die Rolle eines DBMS eingenommen wird
*/
function LmsConnector(
    node        // DOM-Knoten, der für die Benutzeroberfläche genutzt werden soll
) {
    this.upgradeLicenceBtn();
    this._ui = new HumanDbmsTaskUI(node);
    this.connect();
    this._ui.onlevelunlock = (() => this.saveScore());
    this._ui.onnewfine = (() => this.saveScore());
    this._ui.ontablechange = (csv => (this.canSave() && this.write('suspendData', csv)));
}

// Genutzte TaskUI-Instanz
LmsConnector.prototype._ui = null;

/**
* Schaltfläche zum Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
LmsConnector.prototype.upgradeLicenceBtn = function() {
    new ScoTools.InfoPageLink();
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
LmsConnector.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
    }
    if (!this.lmslink.ready) {
        this._ui.setGeneralMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn');
        this._ui.loadValues(LmsConnector.genSeed(), 0, 0);
        return;
    }
    // Werte laden
    try {
        var saved = JSON.parse('[' + this.lmslink.lessonLocation + ']');
        if (saved.length >= 2)
            this._ui.loadValues(+saved[0], +saved[1], +saved[2], this.read('suspendData'));
        else
            this._ui.loadValues(LmsConnector.genSeed(), 0, 0);
    } catch(e) {
        this._ui.loadValues(LmsConnector.genSeed(), 0, 0);
        this._ui.setGeneralMsg('Kommunikationsprobleme mit Lernplattform - Fortschritte können evtl. nicht gespeichert werden', 'msg_warn');
        return;
    }
}

/**
* Zufallszahl generieren, die als festes Start-Seed für die Pseudozufallszahlengenerierung verwendet wird
* @return Number    Seed
*/
LmsConnector.genSeed = function() {
    return Math.floor(Math.random() * 233280);
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
LmsConnector.prototype.lmsValuesInit = function() {
    this.lmslink.lessonLocation = LmsConnector.genSeed() + ',0,0';
}

// Genutzte Instanz von LMSLink
LmsConnector.prototype.lmslink = { 'ready': false, lessonLocation: '1,0,0' };

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
LmsConnector.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
LmsConnector.prototype.canSave = function() {
    return (!this.lmslink.ready || this.lmslink.isNormalIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
LmsConnector.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
LmsConnector.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (!this.canSave()) {
        this.reportTime();
        return;
    }
    this.lmslink.lessonLocation = this._ui.startSeed + ',' + this._ui.unlockedLevelCount + ',' + this._ui.fineCount;
    if (this._ui.done)
        this.lmslink.saveScore(Math.min(100, Math.max(0, 100 - this._ui.fineCount)), 0, 100, "completed");
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
LmsConnector.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Einen Wert vom LMS speichern lassen
* @return undefined
*/
LmsConnector.prototype.write = function(
    key,        // Attributname von LMSLink-Instanz, unter dem der Wert gespeichert werden soll
    value       // Zu speichernder Wert
) {
    if (this.lmslink.ready)
        this.lmslink[key] = value;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
LmsConnector.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    if (this.lmslink.ready)
        this.lmslink.save(data);
}
